@@ -5,22 +5,8 @@ from django.db import models
 from django.contrib.auth.models import Group
 from django.utils.translation import ugettext_lazy as _
 from django.conf import settings
-from django.core.exceptions import AppRegistryNotReady
 from django.utils.encoding import python_2_unicode_compatible
 
-try:
-    # Django 1.7 way for importing custom user
-    from django.contrib.auth import AUTH_USER_MODEL
-    User = get_user_model()
-except ImportError:
-    try:
-        # Django 1.6 way for importing custom user, will crash in Django 1.7
-        from django.contrib.auth import get_user_model
-        User = get_user_model()
-    except AppRegistryNotReady:
-        # If all else fails, import default user model -- please report this bug
-        from django.contrib.auth.models import User
-
 
 @python_2_unicode_compatible
 class Restaurant(models.Model):
@@ -36,30 +22,32 @@ class Restaurant(models.Model):
 
 @python_2_unicode_compatible
 class Order(models.Model):
+    group = models.ForeignKey(Group)
     date = models.DateField(_('date'))
     restaurant = models.ForeignKey(Restaurant)
     extra_costs = models.FloatField(_('extra costs'), default=0)
     active = models.BooleanField(_('Order currently active'), default=True)
     use_validation = models.BooleanField(_('Enable funds validation'), default=True)
 
+    @property
     def get_total_sum(self):
         s = self.orderline_set.aggregate(models.Sum('price'))['price__sum']
         if s is None:
             s = 0
         return s + self.extra_costs
 
+    # Should rename to "get_extra_costs_for_each_user" or -each_payer
     def get_extra_costs(self):
-        orderlines = self.orderline_set.all()
+        # users = self.orderline_set.aggregate(models.Sum('users'))['users__sum']
         users = 0
-        for orderline in orderlines:
-            if orderline.users:
-                users += orderline.users.count()
-            else:
-                users += 1
-        return self.extra_costs / users
+        for ol in self.orderline_set.all():
+            users += ol.users.count()
+        return self.extra_costs / users if users > 0 else self.extra_costs
 
     def order_users(self):
-        return User.objects.filter(groups__name=settings.FEEDME_GROUP)
+        from django.contrib.auth import get_user_model
+        User = get_user_model()
+        return User.objects.filter(groups=self.group)
 
     def available_users(self):
         order_users = self.order_users()
@@ -70,7 +58,8 @@ class Order(models.Model):
     def taken_users(self):
         return self.orderline_set.values_list(_('creator'), flat=True)
 
-    def get_latest(self):
+    @classmethod
+    def get_latest(cls):
         if Order.objects.all():
             orders = Order.objects.all().order_by('-id')
             for order in orders:
@@ -79,46 +68,65 @@ class Order(models.Model):
         else:
             return False
 
+    @property
+    def paid(self):
+        if self.orderline_set.all():
+            for ol in self.orderline_set.all():
+                if not ol.paid_for:
+                    return False
+            return True
+        return False
+
     def __str__(self):
-        return "%s @ %s" % (self.date.strftime("%d-%m-%Y"), self.restaurant)
+        return "%s: %s @ %s" % (self.group, self.date.strftime("%d. %B"), self.restaurant)
 
     class Meta:
         get_latest_by = 'date'
+        permissions = (
+            ('view_order', 'View Order'),
+        )
 
 
 @python_2_unicode_compatible
 class OrderLine(models.Model):
     order = models.ForeignKey(Order)
     creator = models.ForeignKey(settings.AUTH_USER_MODEL, related_name=_('owner'))
-    users = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name=_('buddies'), null=True, blank=True)
-    menu_item = models.IntegerField(_('menu item'), max_length=2)
+    users = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name=_('buddies'), blank=True)
+    menu_item = models.CharField(_('menu item'), max_length=50)
     soda = models.CharField(_('soda'), blank=True, null=True, max_length=25)
     extras = models.CharField(_('extras/comments'), blank=True, null=True, max_length=50)
-    price = models.IntegerField(_('price'), max_length=4)
+    price = models.IntegerField(_('price'))
     paid_for = models.BooleanField(_('paid for'), default=False)
 
     def get_order(self):
         return self.order
 
-    def get_buddies(self):
+    def get_users(self):
         return self.users
 
-    def get_num_users(self):
-        return len(self.users)
+    def get_buddies(self):
+        return self.get_users()
 
+    def get_num_users(self):
+        return self.get_users().count()
+
+    # Should rename to "get_orderline_total"
     def get_total_price(self):
-        return (self.order.get_extra_costs() * self.users.count()) + self.price
+        return self.price + (self.order.get_extra_costs() * self.get_num_users())
+
+    def get_price_to_pay(self):
+        return self.get_total_price() / self.get_num_users() if self.get_num_users() > 0 else 0
 
     def __str__(self):
         return self.creator.get_username()
 
     @models.permalink
     def get_absolute_url(self):
-        return ('edit', (), {'orderline_id': self.id})
+        return 'edit', (), {'orderline_id': self.id}
 
     class Meta:
-        verbose_name = _('Order line')
-        verbose_name_plural = _('Order lines')
+        verbose_name = _('Orderline')
+        verbose_name_plural = _('Orderlines')
 
 
 @python_2_unicode_compatible
@@ -140,6 +148,9 @@ class Balance(models.Model):
             self.add_transaction(0)
         return self.user.transaction_set.aggregate(models.Sum('amount'))['amount__sum']
 
+    def get_balance_string(self):
+        return "%.2f kr" % self.get_balance()
+
     def add_transaction(self, amount):
         transaction = Transaction()
         transaction.user = self.user
@@ -156,12 +167,12 @@ class Balance(models.Model):
         # print('Deprecated notice, please add new transaction objects rather than calling the Balance object')
 
     def __str__(self):
-        return "%s: %s" % (self.user, self.get_balance())
-
+        return "%s: %s" % (self.user.username, self.get_balance())
 
 
 @python_2_unicode_compatible
 class Poll(models.Model):
+    group = models.ForeignKey(Group)
     question = models.CharField(_('question'), max_length=250)
     active = models.BooleanField(_('active'), default=True)
     due_date = models.DateTimeField(_('due date'))
@@ -177,7 +188,9 @@ class Poll(models.Model):
         # Throw some exception if fails?
 
     @classmethod
-    def get_active(self):
+    def get_active(cls):
+        if Poll.objects.count() == 0:
+            return None
         if Poll.objects.latest('id').active:
             return Poll.objects.latest('id')
         else:
@@ -193,7 +206,7 @@ class Poll(models.Model):
         return r
 
     def get_winner(self):
-        winner = (None,-1)
+        winner = (None, -1)
         results = self.get_result()
         for key in results:
             if results[key] > winner[1]:
@@ -201,18 +214,28 @@ class Poll(models.Model):
         return winner[0]
 
     def __str__(self):
-        return self.question
+        return "%s due %s" % (self.question, self.due_date.strftime("%x"))
+
+    class Meta:
+        permissions = (
+            ('view_poll', 'View Poll'),
+        )
 
 
 @python_2_unicode_compatible
 class Answer(models.Model):
     user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name=_('user'))
-    poll = models.ForeignKey(Poll, related_name=_('poll'))
+    poll = models.ForeignKey(Poll, related_name=_('votes'))
     answer = models.ForeignKey(Restaurant, related_name=_('answer'))
 
     def __str__(self):
         return "%s: %s (%s)" % (self.user, self.answer, self.poll)
 
+    class Meta:
+        permissions = (
+            ('vote_poll', 'Vote Poll'),
+        )
+
 
 class ManageBalance(models.Model):
     user = models.ForeignKey(Balance)
