@@ -4,6 +4,6 @@ from django import template
 
 register = template.Library()
 
-@register.inclusion_tag('admin_tabs.html')
+@register.inclusion_tag('feedme/admin_tabs.html')
 def admin_tabs(active):
     return {'active': active}
